@@ -33,10 +33,10 @@
 #include "krandom.h"
 #include "kshell.h"
 #include "kmessagebox.h" // TODO get rid of message box
-#include <qdesktopservices.h> // TODO remove, this was only for my playing round
+#include <KRun>
 #include <qstandardpaths.h>
 #include "klocalizedstring.h"
-#include "QDebug"
+#include <knewstuff_debug.h>
 
 #include "core/security_p.h"
 #ifdef Q_OS_WIN
@@ -229,7 +229,7 @@ void Installation::slotPayloadResult(KJob *job)
                 if (mimeType.inherits(QStringLiteral("text/html")) || mimeType.inherits(QStringLiteral("application/x-php"))) {
                     if (KMessageBox::questionYesNo(0, i18n("The downloaded file is a html file. This indicates a link to a website instead of the actual download. Would you like to open the site with a browser instead?"), i18n("Possibly bad download link"))
                             == KMessageBox::Yes) {
-                        QDesktopServices::openUrl(fcjob->srcUrl());
+                        KRun::runUrl(fcjob->srcUrl(), QStringLiteral("text/html"), Q_NULLPTR);
                         emit signalInstallationFailed(i18n("Downloaded file was a HTML file. Opened in browser."));
                         entry.setStatus(Entry::Invalid);
                         emit signalEntryChanged(entry);
@@ -300,7 +300,6 @@ void Installation::install(KNS3::EntryInternal entry, const QString &downloadedF
     entry.setInstalledFiles(installedFiles);
 
     if (!postInstallationCommand.isEmpty()) {
-        QString target;
         if (installedFiles.size() == 1) {
             runPostInstallationCommand(installedFiles.first());
         } else {
@@ -363,8 +362,6 @@ QString Installation::targetInstallationPath(const QString &payloadfile)
             //crude translation KStandardDirs names -> QStandardPaths enum
             if (standardResourceDirectory == QLatin1String("tmp")) {
                 location = QStandardPaths::TempLocation;
-            } else if (standardResourceDirectory == QLatin1String("data")) {
-                location = QStandardPaths::GenericDataLocation;
             } else if (standardResourceDirectory == QLatin1String("config")) {
                 location = QStandardPaths::ConfigLocation;
             }
@@ -376,16 +373,16 @@ QString Installation::targetInstallationPath(const QString &payloadfile)
             }
             pathcounter++;
         }
-        if (!targetDirectory.isEmpty()) {
+        if (!targetDirectory.isEmpty() && targetDirectory != "/") {
             if (scope == ScopeUser) {
-                installdir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1Char('/') + targetDirectory + '/';
+                installdir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1Char('/') + targetDirectory + QLatin1Char('/');
             } else { // system scope
-                installdir = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).last() + '/' + targetDirectory + '/';
+                installdir = QStandardPaths::locate(QStandardPaths::GenericDataLocation, targetDirectory, QStandardPaths::LocateDirectory) + QLatin1Char('/');
             }
             pathcounter++;
         }
-        if (!xdgTargetDirectory.isEmpty()) {
-            installdir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1Char('/') + xdgTargetDirectory + '/';
+        if (!xdgTargetDirectory.isEmpty() && xdgTargetDirectory != "/") {
+            installdir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1Char('/') + xdgTargetDirectory + QLatin1Char('/');
             pathcounter++;
         }
         if (!installPath.isEmpty()) {
@@ -397,12 +394,12 @@ QString Installation::targetInstallationPath(const QString &payloadfile)
                 installdir =  QDir::home().path() + QLatin1Char('/') + installPath + QLatin1Char('/');
             }
 #else
-            installdir = QDir::home().path() + '/' + installPath + '/';
+            installdir = QDir::home().path() + QLatin1Char('/') + installPath + QLatin1Char('/');
 #endif
             pathcounter++;
         }
         if (!absoluteInstallPath.isEmpty()) {
-            installdir = absoluteInstallPath + '/';
+            installdir = absoluteInstallPath + QLatin1Char('/');
             pathcounter++;
         }
 
@@ -414,11 +411,7 @@ QString Installation::targetInstallationPath(const QString &payloadfile)
         qCDebug(KNEWSTUFF) << "installdir: " << installdir;
 
         // create the dir if it doesn't exist (QStandardPaths doesn't create it, unlike KStandardDirs!)
-        QDir dir(installdir);
-        if (!dir.exists(installdir)) {
-            dir.mkpath(installdir);
-        }
-
+        QDir().mkpath(installdir);
     }
 
     return installdir;
@@ -443,10 +436,10 @@ QStringList Installation::installDownloadedFileAndUncompress(const KNS3::EntryIn
 
             // FIXME: check for overwriting, malicious archive entries (../foo) etc.
             // FIXME: KArchive should provide "safe mode" for this!
-            KArchive *archive = 0;
+            QScopedPointer<KArchive> archive;
 
             if (mimeType.inherits(QStringLiteral("application/zip"))) {
-                archive = new KZip(payloadfile);
+                archive.reset(new KZip(payloadfile));
             } else if (mimeType.inherits(QStringLiteral("application/tar"))
                        || mimeType.inherits(QStringLiteral("application/x-gzip"))
                        || mimeType.inherits(QStringLiteral("application/x-bzip"))
@@ -454,9 +447,8 @@ QStringList Installation::installDownloadedFileAndUncompress(const KNS3::EntryIn
                        || mimeType.inherits(QStringLiteral("application/x-xz"))
                        || mimeType.inherits(QStringLiteral("application/x-bzip-compressed-tar"))
                        || mimeType.inherits(QStringLiteral("application/x-compressed-tar"))) {
-                archive = new KTar(payloadfile);
+                archive.reset(new KTar(payloadfile));
             } else {
-                delete archive;
                 qCritical() << "Could not determine type of archive file '" << payloadfile << "'";
                 if (uncompression == QLatin1String("always")) {
                     return QStringList();
@@ -477,14 +469,18 @@ QStringList Installation::installDownloadedFileAndUncompress(const KNS3::EntryIn
 
                 if (isarchive) {
                     const KArchiveDirectory *dir = archive->directory();
-                    dir->copyTo(installdir);
+                    //if there is more than an item in the file,
+                    //put contents in a subdirectory with the same name as the file
+                    if (dir->entries().count() > 1) {
+                        installpath = installdir + QLatin1Char('/') + QFileInfo(archive->fileName()).baseName();
+                    }
+                    dir->copyTo(installpath);
 
-                    installedFiles << archiveEntries(installdir, dir);
-                    installedFiles << installdir + '/';
+                    installedFiles << archiveEntries(installpath, dir);
+                    installedFiles << installpath + QLatin1Char('/');
 
                     archive->close();
                     QFile::remove(payloadfile);
-                    delete archive;
                 }
             }
         }
@@ -521,7 +517,7 @@ QStringList Installation::installDownloadedFileAndUncompress(const KNS3::EntryIn
                     installfile = source.fileName();
                 }
             }
-            installpath = installdir + '/' + installfile;
+            installpath = installdir + QLatin1Char('/') + installfile;
 
             qCDebug(KNEWSTUFF) << "Install to file " << installpath;
             // FIXME: copy goes here (including overwrite checking)
@@ -638,7 +634,7 @@ QStringList Installation::archiveEntries(const QString &path, const KArchiveDire
 {
     QStringList files;
     foreach (const QString &entry, dir->entries()) {
-        QString childPath = path + '/' + entry;
+        QString childPath = path + QLatin1Char('/') + entry;
         if (dir->entry(entry)->isFile()) {
             files << childPath;
         }
@@ -646,7 +642,7 @@ QStringList Installation::archiveEntries(const QString &path, const KArchiveDire
         if (dir->entry(entry)->isDirectory()) {
             const KArchiveDirectory *childDir = static_cast<const KArchiveDirectory *>(dir->entry(entry));
             files << archiveEntries(childPath, childDir);
-            files << childPath + '/';
+            files << childPath + QLatin1Char('/');
         }
     }
     return files;
